@@ -15,7 +15,6 @@ import qualified Git.DiffTree as DiffTree
 import Git.Sha
 import Git.Types
 import Annex.CatFile
-import Utility.FileMode
 import qualified Annex.Queue
 import Logs.Location
 import Backend
@@ -45,8 +44,8 @@ stageDirect = do
 	{- Determine what kind of modified or deleted file this is, as
 	 - efficiently as we can, by getting any key that's associated
 	 - with it in git, as well as its stat info. -}
-	go (file, Just sha) = do
-		shakey <- catKey sha
+	go (file, Just sha, Just mode) = do
+		shakey <- catKey sha mode
 		mstat <- liftIO $ catchMaybeIO $ getSymbolicLinkStatus file
 		filekey <- isAnnexLink file
 		case (shakey, filekey, mstat, toInodeCache =<< mstat) of
@@ -147,10 +146,9 @@ mergeDirectCleanup d oldsha newsha = do
 	  where
 		go getsha getmode a araw
 			| getsha item == nullSha = noop
-			| isSymLink (getmode item) =
+			| otherwise =
 				maybe (araw f) (\k -> void $ a k f)
-					=<< catKey (getsha item)
-			| otherwise = araw f
+					=<< catKey (getsha item) (getmode item)
 		f = DiffTree.file item
 
 	moveout = removeDirect
