@@ -48,7 +48,10 @@ buildFlags = filter (not . null)
 #ifdef WITH_DNS
 	, "DNS"
 #endif
-#ifdef WITH_FEEDS
+#ifdef WITH_FEED
 	, "Feeds"
+#endif
+#ifdef WITH_QUVI
+	, "Quvi"
 #endif
 	]
