@@ -32,9 +32,8 @@ perform key = maybe droplocal dropremote =<< Remote.byNameWithUUID =<< from
   where
 	dropremote r = do
 		showAction $ "from " ++ Remote.name r
-		ok <- Remote.removeKey r key
-		next $ Command.Drop.cleanupRemote key r ok
-	droplocal = Command.Drop.performLocal key (Just 0) Nothing -- force drop
+		Command.Drop.performRemote key Nothing r
+	droplocal = Command.Drop.performLocal key Nothing Nothing
 	from = Annex.getField $ Option.name Command.Drop.fromOption
 
 performOther :: (Key -> Git.Repo -> FilePath) -> Key -> CommandPerform
