@@ -7,18 +7,21 @@
 
 module Git.Index where
 
-import System.Posix.Env (setEnv, unsetEnv, getEnv)
+import Utility.Env
 
 {- Forces git to use the specified index file.
  -
  - Returns an action that will reset back to the default
- - index file. -}
+ - index file.
+ -
+ - Warning: Not thread safe.
+ -}
 override :: FilePath -> IO (IO ())
 override index = do
 	res <- getEnv var
 	setEnv var index True
 	return $ reset res
-	where
-		var = "GIT_INDEX_FILE"
-		reset (Just v) = setEnv var v True
-		reset _ = unsetEnv var
+  where
+	var = "GIT_INDEX_FILE"
+	reset (Just v) = setEnv var v True
+	reset _ = unsetEnv var
