@@ -10,8 +10,7 @@ module Git.UnionMerge (
 	mergeIndex
 ) where
 
-import qualified Data.Text.Lazy as L
-import qualified Data.Text.Lazy.Encoding as L
+import qualified Data.ByteString.Lazy as L
 import qualified Data.Set as S
 
 import Common
@@ -39,10 +38,16 @@ merge x y repo = do
 	catFileStop h
 
 {- Merges a list of branches into the index. Previously staged changes in
- - the index are preserved (and participate in the merge). -}
+ - the index are preserved (and participate in the merge).
+ -
+ - update-index is run once per ref in turn, so that each ref is merged on
+ - top of the merge for the previous ref. It would be more efficient, but
+ - harder to calculate a single union merge involving all the refs, as well
+ - as the index.
+ -}
 mergeIndex :: CatFileHandle -> Repo -> [Ref] -> IO ()
-mergeIndex h repo bs =
-	streamUpdateIndex repo $ map (\b -> mergeTreeIndex b h repo) bs
+mergeIndex h repo bs = forM_ bs $ \b ->
+	streamUpdateIndex repo [mergeTreeIndex b h repo]
 
 {- For merging two trees. -}
 mergeTrees :: Ref -> Ref -> CatFileHandle -> Repo -> Streamer
@@ -59,13 +64,15 @@ diffOpts = ["--raw", "-z", "-r", "--no-renames", "-l0"]
 {- Streams update-index changes to perform a merge,
  - using git to get a raw diff. -}
 doMerge :: CatFileHandle -> [String] -> Repo -> Streamer
-doMerge ch differ repo streamer = gendiff >>= go
-	where
-		gendiff = pipeNullSplit (map Param differ) repo
-		go [] = noop
-		go (info:file:rest) = mergeFile info file ch repo >>=
-			maybe (go rest) (\l -> streamer l >> go rest)
-		go (_:[]) = error $ "parse error " ++ show differ
+doMerge ch differ repo streamer = do
+	(diff, cleanup) <- pipeNullSplit (map Param differ) repo
+	go diff
+	void $ cleanup
+  where
+	go [] = noop
+	go (info:file:rest) = mergeFile info file ch repo >>=
+		maybe (go rest) (\l -> streamer l >> go rest)
+	go (_:[]) = error $ "parse error " ++ show differ
 
 {- Given an info line from a git raw diff, and the filename, generates
  - a line suitable for update-index that union merges the two sides of the
@@ -77,12 +84,16 @@ mergeFile info file h repo = case filter (/= nullSha) [Ref asha, Ref bsha] of
 	shas -> use
 		=<< either return (\s -> hashObject BlobObject (unlines s) repo)
 		=<< calcMerge . zip shas <$> mapM getcontents shas
-	where
-		[_colonmode, _bmode, asha, bsha, _status] = words info
-		getcontents s = map L.unpack . L.lines .
-			L.decodeUtf8 <$> catObject h s
-		use sha = return $ Just $
-			updateIndexLine sha FileBlob $ asTopFilePath file
+  where
+	[_colonmode, _bmode, asha, bsha, _status] = words info
+	use sha = return $ Just $
+		updateIndexLine sha FileBlob $ asTopFilePath file
+	-- We don't know how the file is encoded, but need to
+	-- split it into lines to union merge. Using the
+	-- FileSystemEncoding for this is a hack, but ensures there
+	-- are no decoding errors. Note that this works because
+	-- hashObject sets fileEncoding on its write handle.
+	getcontents s = lines . encodeW8 . L.unpack <$> catObject h s
 
 {- Calculates a union merge between a list of refs, with contents.
  -
@@ -93,7 +104,7 @@ calcMerge :: [(Ref, [String])] -> Either Ref [String]
 calcMerge shacontents
 	| null reuseable = Right $ new
 	| otherwise = Left $ fst $ Prelude.head reuseable
-	where
-		reuseable = filter (\c -> sorteduniq (snd c) == new) shacontents
-		new = sorteduniq $ concat $ map snd shacontents
-		sorteduniq = S.toList . S.fromList
+  where
+	reuseable = filter (\c -> sorteduniq (snd c) == new) shacontents
+	new = sorteduniq $ concat $ map snd shacontents
+	sorteduniq = S.toList . S.fromList
