@@ -18,6 +18,7 @@ import Types.UUID
 import Types.GitConfig
 import Config.Cost
 import Utility.Metered
+import Git.Remote
 
 type RemoteConfigKey = String
 type RemoteConfig = M.Map RemoteConfigKey String
@@ -29,9 +30,9 @@ data RemoteTypeA a = RemoteType {
 	-- enumerates remotes of this type
 	enumerate :: a [Git.Repo],
 	-- generates a remote of this type
-	generate :: Git.Repo -> UUID -> RemoteConfig -> RemoteGitConfig -> a (RemoteA a),
+	generate :: Git.Repo -> UUID -> RemoteConfig -> RemoteGitConfig -> a (Maybe (RemoteA a)),
 	-- initializes or changes a remote
-	setup :: UUID -> RemoteConfig -> a RemoteConfig
+	setup :: Maybe UUID -> RemoteConfig -> a (RemoteConfig, UUID)
 }
 
 instance Eq (RemoteTypeA a) where
@@ -42,7 +43,7 @@ data RemoteA a = Remote {
 	-- each Remote has a unique uuid
 	uuid :: UUID,
 	-- each Remote has a human visible name
-	name :: String,
+	name :: RemoteName,
 	-- Remotes have a use cost; higher is more expensive
 	cost :: Cost,
 	-- Transfers a key to the remote.
